#INCLUDE CRASH82.INC
.DB "Diamond v1.0 by HC", 0

lives = TEXT_MEM              ; Number of lives
level = TEXT_MEM+1            ; The current level < 26
score = TEXT_MEM+2            ; The score

  LD  A, $8C                  ; Swap to ROM page 4
  OUT (2), A                  ; so FIND_PIXEL can be used

  LD  HL, lives               ; Initialise variables:
  LD  (HL), 5                 ; Lives = 5
  INC HL
  LD  (HL), 1                 ; Level = 1
  INC HL
  LD  (HL), 0                 ; Score = 0

DrawStart:                    ; Start by drawing border (and clearing screen)
  LD  HL, GRAPH_MEM           ; Top line
  LD  DE, GRAPH_MEM+1
  LD  (HL), $FF
  LD  BC, 12
  LDIR

  PUSH HL                     ; Clear screen (except top and bottom rows)
  LD  (HL), $00
  LD  BC, 744
  LDIR

  LD  (HL), $FF               ; Bottom line
  LD  BC, 11
  LDIR

  POP HL                      ; Vertical lines
  LD  B, 62                   ; two pixels at a time
  LD  DE, 11
LoopVert:
  LD  (HL), $80
  ADD HL, DE
  LD  (HL), $01
  INC HL
  DJNZ LoopVert               ; End of border routine

  LD  DE, (level)             ; E will be counter for number of diamonds to draw
  SLA E                       ; Multiply level by 10 using 2x + 8x = 10x
  LD  A, E
  SLA A
  SLA A
  ADD A, E
  LD  E, A

DrawDiamond:                  ; Now draw the diamonds
  CALL RAND                   ; Randomly determine x-coord
  CP  90
  JR  NC, DrawDiamond         ; 0 <= A <= 89
  INC A
  LD  B, A                    ; B contains x-coord
LoopRandY:
  CALL RAND                   ; Randomly determine y-coord
  AND %00111111               ; Mask out lower 6 bits
  CP  58
  JR  NC, LoopRandY           ; 0 <= A <= 57
  ADD A, 5                    ; y is inverted in the call to FIND_PIXEL
  LD  C, A                    ; C contains y-coord

  LD  D, 5                    ; D is counter for number of columns to draw
  LD  IX, DiamondImage        ; IX is address of DiamondImage data

LoopDiamondCol:
  PUSH DE                     ; Save diamond and column counters
  PUSH BC                     ; FIND_PIXEL destroys BC!
  CALL FIND_PIXEL
  LD  DE, GRAPH_MEM
  ADD HL, DE                  ; Add GRAPH_MEM to offset
  LD  D, 0
  LD  E, (IX+0)               ; LD  DE, (IX)
  INC IX                      ; the row offset * 12
  ADD HL, DE                  ; Add row offset to GRAPH_MEM address
  LD  DE, 12
  LD  B, (IX+0)               ; Number of consecutive pixels (rows) in column
  INC IX
  LD  C, A                    ; Save bit set mask for multiple use
LoopDiamondRow:
  OR  (HL)
  LD  (HL), A                 ; Set a pixel in column of diamond
  LD  A, C
  ADD HL, DE                  ; Move to next row
  DJNZ LoopDiamondRow         ; Decrement row counter and test
  POP BC
  INC B
  POP DE
  DEC D                       ; Decrement column counter
  JR  NZ, LoopDiamondCol      ; and test

  DEC E                       ; Decrement diamond counter and test
  JR  NZ, DrawDiamond         ; End of diamond routine

  CALL CR_GRBCopy             ; Finally, display
  LD  B, 100                  ; Small delay before start
StartDelay:
  HALT
  DJNZ StartDelay

  LD  HL, GRAPH_MEM+12        ; Move to second row
  LD  BC, 12
  LD  D, $40                  ; Bit set mask set to pixel 2 of 8 in byte
  LD  E, 0                    ; Counts the number of pixels travelled

DrawPlayer:                   ; Start of player draw routine
  LD  A, D                    ; Test for existing pixel
  AND (HL)
  JR  NZ, GameOver            ; Game over if pixel was previously set
  LD  A, D                    ; else draw a player pixel
  OR  (HL)
  LD  (HL), A
  CALL CR_GRBCopy             ; Display
  INC E                       ; Increase pixel counter
  SRL D                       ; Shift to next pixel in byte
  JR  NZ, GetKey              ; Skip over to GetKey if not end of byte yet
  INC HL                      ; Move to next byte in GRAPH_MEM
  LD  D, $80                  ; Reset bit set mask to pixel 1 of 8 in byte
GetKey:
  LD  B, 6                    ; Delay reading of input
GetKeyDelay:                  ; to allow player to react
  HALT
  DJNZ GetKeyDelay
  LD  A, 0                    ; Check all keys
  OUT (1), A
  IN  A, (1)
  INC A                       ; $FF if no keys pressed
  JR  NZ, MoveUp              ; If keys pressed skip to move up routine
  ADD HL, BC                  ; Move down one row
  JR  DrawPlayer              ; Return to top of loop
MoveUp:
  OR  A                       ; Clear carry for SBC
  SBC HL, BC                  ; Move up one row
  JR  DrawPlayer              ; End of player draw routine

GameOver:                     ; Player either won or died trying
  LD  B, 100                  ; Small delay before CR_KHAND
FinalDelay:
  HALT
  DJNZ FinalDelay
  PUSH DE                     ; CR_KHAND destroys everything!
  CALL CR_KHAND
  POP DE
  LD  HL, (score)             ; Add pixels travelled to score
  LD  D, 0                    ; Clear D for ADD
  ADD HL, DE
  DEC HL                      ; Subtract free point gained from start
  LD  (score), HL             ; Save new score to memory
  LD  A, E                    ; If at far right of screen,
  CP  94                      ; start a new level
  JR  C, LoseLife
  LD  HL, (level)
  INC L                       ; Level-up
  LD  (level), HL             ; Save the new level in memory
  JP  DrawStart
LoseLife:
  LD  HL, (lives)             ; else lose a life
  DEC L                       ; and restart the level
  LD  (lives), HL             ; (save lives in memory first)
  JP  NZ, DrawStart           ; If no lives left then exit

  LD  HL, $0203               ; Set cursor coords (2,3)
  LD  (CURSOR_POS), HL
  SET 3, (IY+5)               ; Display inverted text
  LD  HL, ScoreStr
  ROM_CALL(D_ZT_STR)          ; Write "Score: "
  LD  HL, (score)
  ROM_CALL(D_HL_DECI)         ; and the actual score next to it
  JP  CR_KHAND                ; Pause, then return to CrASH

DiamondImage:                 ; Row offset * 12, Number of consecutive pixels
  .DB 24, 1, 12, 3, 0, 5, 12, 3, 24, 1
ScoreStr:
  .DB "Score: ", 0
